#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "complex.h"
#include "landen_trans.h"
#define MAX_ITER 12

/*   . */
double complete_intgr(double k)
{
	double k_next, K;
	double sqr;
	
	sqr = sqrt(1.0 - k * k);
	k_next = pow(k / (1.0 + sqr), 2.0);
	
	K = M_PI / 2.0;
	
	while ((k - k_next) > 0.00001) {
		k = k_next;
		K *= 1.0 + k_next;
		sqr = sqrt(1.0 - k * k);
		k_next = pow(k / (1.0 + sqr), 2.0);
	}
	
	return K;
}		

/*    . */		
double complement_intgr(double k)
{	
	double k_next, K;
	double sqr;
	
	k = sqrt(1.0 - k * k);
	
	sqr = sqrt(1.0 - k * k);
	k_next = pow(k / (1.0 + sqr), 2.0);
	
	K = M_PI / 2.0;
	
	while ((k - k_next) > 0.00001) {
		k = k_next;
		K *= 1.0 + k_next;
		sqr = sqrt(1.0 - k * k);
		k_next = pow(k / (1.0 + sqr), 2.0);
	}
	
	return K;
}				

/*  . */
complex_t sn(complex_t u, double k)
{
	int i;
	double sqr;
	double k_coeff[MAX_ITER];
	complex_t one, sum_one_k, compl_k;
	complex_t sn_i;

	for (i = 0; i < MAX_ITER; i++) {
		sqr = sqrt(1.0 - k * k);
		k = pow(k / (1.0 + sqr), 2.0);
		k_coeff[i] = k;
	}
	
	one = c_num(1.0, 0.0);
	sn_i = c_sin(c_mult(u, c_num(M_PI / 2.0, 0.0)));
    
	for (i = MAX_ITER - 1; i >= 0; i--) {
		sum_one_k = c_num(1.0 + k_coeff[i], 0.0);
		compl_k = c_num(k_coeff[i], 0.0);
		sn_i = c_div(sum_one_k, c_sum(c_div(one, sn_i), c_mult(compl_k, sn_i)));
	}
	
	return sn_i;
}

/*    cn/dn.
 (cn -  , dn = sqrt(1-k^2*sn^2) */
complex_t cd(complex_t u, double k)
{
	int i;
	double sqr;
	double k_coeff[MAX_ITER];
	complex_t one, sum_one_k, compl_k;
	complex_t cd_i;

	for (i = 0; i < MAX_ITER; i++) {
		sqr = sqrt(1.0 - k * k);
		k = pow(k / (1.0 + sqr), 2.0);
		k_coeff[i] = k;
	}

	one = c_num(1.0, 0.0);
	cd_i = c_cos(c_mult(u, c_num(M_PI / 2.0, 0.0)));

	for (i = MAX_ITER - 1; i >= 0; i--) {
		sum_one_k = c_num(1.0 + k_coeff[i], 0.0);
		compl_k = c_num(k_coeff[i], 0.0);
		cd_i = c_div(sum_one_k, c_sum(c_div(one, cd_i), c_mult(compl_k, cd_i)));
	}
	
	return cd_i;
}

/*   cd. */
complex_t cd_inverse(complex_t w, double k)
{
	int i;
	double sqr;
	double k_coeff[MAX_ITER+1];
	complex_t one, two;
	complex_t sqr_compl, u_square, k_square, sum_one_knext, sum_one_sqrt;
	complex_t u, res;
    
	for (i = 0; i < MAX_ITER + 1; i++) {
		k_coeff[i] = k;
		sqr = sqrt(1.0 - k * k);
		k = pow(k / (1.0 + sqr), 2.0);
	}

	u = w;
	one = c_num(1.0, 0.0);
	two = c_num(2.0, 0.0);
	
	for (i = 0; i < MAX_ITER; i++){
		k_square = c_num(k_coeff[i] * k_coeff[i], 0.0);
		u_square = c_mult(u, u);
		sqr_compl = c_sqrt(c_sub(one, c_mult(k_square, u_square)));
		sum_one_knext = c_num(1.0 + k_coeff[i + 1], 0.0);
		sum_one_sqrt = c_sum(one, sqr_compl);
		
		u = c_div(c_mult(two, u), c_mult(sum_one_knext, sum_one_sqrt));
	}

	res = c_mult(c_num(2.0 / M_PI, 0.0), c_acos(u));
	return res;
}

/*   sn. */
complex_t sn_inverse(complex_t w, double k)
{
	int i;
	double sqr;
	double k_coeff[MAX_ITER + 1];
	complex_t one, two;
	complex_t sqr_compl, u_square, k_square, sum_one_knext, sum_one_sqrt;
	complex_t u, res;

	for (i = 0; i < MAX_ITER + 1; i++) {
		k_coeff[i] = k;
		sqr = sqrt(1.0 - k * k);
		k = pow(k / (1.0 + sqr), 2.0);
	}

	u = w;
	one = c_num(1.0, 0.0);
	two = c_num(2.0, 0.0);
	
	for (i = 0; i < MAX_ITER; i++){
		k_square = c_num(k_coeff[i] * k_coeff[i], 0.0);
		u_square = c_mult(u, u);
		sqr_compl = c_sqrt(c_sub(one, c_mult(k_square, u_square)));
		sum_one_knext = c_num(1.0 + k_coeff[i + 1], 0.0);
		sum_one_sqrt = c_sum(one, sqr_compl);
		
		u = c_div(c_mult(two, u), c_mult(sum_one_knext, sum_one_sqrt));
	}

	res = c_mult(c_num(2.0 / M_PI, 0.0), c_asin(u));
	return res;
}

/*   k. */
double recalc_k(int n, double k1)
{	
	double k1_p, k_p, k;
	complex_t u, c;
	int i;

	k1_p = sqrt(1.0 - k1 * k1);
	k_p = pow(k1_p, n);

	for(i = 1; i <= n / 2; i++) {
		u = c_num((2.0 * i - 1.0) / n, 0.0);
		c = sn(u, k1_p);
		k_p *= pow(c.re, 4.0);
	}

	k = sqrt(1.0 - k_p * k_p);
	return k;
}
